/*
 * Jalview - A Sequence Alignment Editor and Viewer ($$Version-Rel$$)
 * Copyright (C) $$Year-Rel$$ The Jalview Authors
 * 
 * This file is part of Jalview.
 * 
 * Jalview is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *  
 * Jalview is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jalview.  If not, see <http://www.gnu.org/licenses/>.
 * The Jalview Authors are detailed in the 'AUTHORS' file.
 */
package jalview.bin;

import jalview.gui.JvOptionPane;
import jalview.ws.utils.Worker;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import org.testng.Assert;
import org.testng.FileAssert;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeTest;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

public class CommandLineOperations
{

  @BeforeClass(alwaysRun = true)
  public void setUpJvOptionPane()
  {
    JvOptionPane.setInteractiveMode(false);
    JvOptionPane.setMockResponse(JvOptionPane.CANCEL_OPTION);
  }

  private static final int TEST_TIMEOUT = 9000; // Note longer timeout needed on
                                                // full test run than on
                                                // individual tests

  private static final int SETUP_TIMEOUT = 9000;

  private static final int MINFILESIZE_SMALL = 2096;

  private static final int MINFILESIZE_BIG = 4096;

  private ArrayList<String> successfulCMDs = new ArrayList<>();


  @BeforeTest(alwaysRun = true)
  public void initialize()
  {
    new CommandLineOperations();
  }

  @BeforeTest(alwaysRun = true)
  public void setUpForHeadlessCommandLineInputOperations()
          throws IOException
  {
    String cmds = "nodisplay -open examples/uniref50.fa -sortbytree -props FILE -colour zappo "
            + "-jabaws http://www.compbio.dundee.ac.uk/jabaws -nosortbytree -dasserver nickname=www.test.com "
            + "-features examples/testdata/plantfdx.features -annotations examples/testdata/plantfdx.annotations -tree examples/testdata/uniref50_test_tree";
    Worker worker = Worker.jalviewDesktopRunner(true, cmds, SETUP_TIMEOUT);
    String ln = null;
    while ((ln = worker.getOutputReader().readLine()) != null)
    {
      System.out.println(ln);
      successfulCMDs.add(ln);
    }
  }

  @BeforeTest(alwaysRun = true)
  public void setUpForCommandLineInputOperations() throws IOException
  {
    String cmds = "-open examples/uniref50.fa -noquestionnaire -nousagestats";
    Worker worker = Worker.jalviewDesktopRunner(false, cmds, SETUP_TIMEOUT);
    String ln = null;
    int count = 0;
    while ((ln = worker.getErrorReader().readLine()) != null)
    {
      System.out.println(ln);
      successfulCMDs.add(ln);
      if (++count > 5)
      {
        break;
      }
    }
    if (worker != null && worker.getExitValue() == null)
    {
      worker.destroyProcess();

    }
  }

  @Test(groups = { "Functional" }, dataProvider = "allInputOpearationsData")
  public void testAllInputOperations(String expectedString,
          String failureMsg)
  {
    Assert.assertTrue(successfulCMDs.contains(expectedString), failureMsg);
  }

  @Test(
    groups = { "Functional" },
    dataProvider = "headlessModeOutputOperationsData")
  public void testHeadlessModeOutputOperations(String harg, String type,
          String fileName, boolean withAWT, int expectedMinFileSize,
          int timeout)
  {
    String cmd = harg + type + " " + fileName;
    // System.out.println(">>>>>>>>>>>>>>>> Command : " + cmd);
    File file = new File(fileName);
    Worker worker = Worker.jalviewDesktopRunner(withAWT, cmd, timeout);

    FileAssert.assertFile(file, "Didn't create an output" + type
            + " file.[" + harg + "]");
    FileAssert.assertMinLength(new File(fileName), expectedMinFileSize);
    if (worker != null && worker.getExitValue() == null)
    {
      worker.destroyProcess();
      Assert.fail("Jalview did not exit after "
              + type
              + " generation (try running test again to verify - timeout at "
              + SETUP_TIMEOUT + "ms). ["
              + harg + "]");
    }
    new File(fileName).delete();
  }

  @DataProvider(name = "allInputOpearationsData")
  public Object[][] getHeadlessModeInputParams()
  {
    return new Object[][] {
        // headless mode input operations
        { "CMD [-color zappo] executed successfully!",
            "Failed command : -color zappo" },
        { "CMD [-props FILE] executed successfully!",
            "Failed command : -props File" },
        { "CMD [-sortbytree] executed successfully!",
            "Failed command : -sortbytree" },
        {
            "CMD [-jabaws http://www.compbio.dundee.ac.uk/jabaws] executed successfully!",
            "Failed command : -jabaws http://www.compbio.dundee.ac.uk/jabaws" },
        { "CMD [-open examples/uniref50.fa] executed successfully!",
            "Failed command : -open examples/uniref50.fa" },
        { "CMD [-nosortbytree] executed successfully!",
            "Failed command : -nosortbytree" },
        { "CMD [-dasserver nickname=www.test.com] executed successfully!",
            "Failed command : -dasserver nickname=www.test.com" },
        {
            "CMD [-features examples/testdata/plantfdx.features]  executed successfully!",
            "Failed command : -features examples/testdata/plantfdx.features" },
        {
            "CMD [-annotations examples/testdata/plantfdx.annotations] executed successfully!",
            "Failed command : -annotations examples/testdata/plantfdx.annotations" },
        {
            "CMD [-tree examples/testdata/uniref50_test_tree] executed successfully!",
            "Failed command : -tree examples/testdata/uniref50_test_tree" },
        // non headless mode input operations
        { "CMD [-nousagestats] executed successfully!",
            "Failed command : -nousagestats" },
        { "CMD [-noquestionnaire] executed successfully!",
            "Failed command : -noquestionnaire nickname=www.test.com" } };

  }

  @DataProvider(name = "headlessModeOutputOperationsData")
  public static Object[][] getHeadlessModeOutputParams()
  {
    return new Object[][] {
        { "nodisplay -open examples/uniref50.fa", " -eps",
            "test_uniref50_out.eps", true, MINFILESIZE_BIG, TEST_TIMEOUT },
        { "nodisplay -open examples/uniref50.fa", " -eps",
            "test_uniref50_out.eps", false, MINFILESIZE_BIG, TEST_TIMEOUT },
        { "nogui -open examples/uniref50.fa", " -eps",
            "test_uniref50_out.eps", true, MINFILESIZE_BIG, TEST_TIMEOUT },
        { "nogui -open examples/uniref50.fa", " -eps",
            "test_uniref50_out.eps", false, MINFILESIZE_BIG, TEST_TIMEOUT },
        { "headless -open examples/uniref50.fa", " -eps",
            "test_uniref50_out.eps", true, MINFILESIZE_BIG, TEST_TIMEOUT },
        { "headless -open examples/uniref50.fa", " -svg",
            "test_uniref50_out.svg", false, MINFILESIZE_BIG, TEST_TIMEOUT },
        { "headless -open examples/uniref50.fa", " -png",
            "test_uniref50_out.png", true, MINFILESIZE_BIG, TEST_TIMEOUT },
        { "headless -open examples/uniref50.fa", " -html",
            "test_uniref50_out.html", true, MINFILESIZE_BIG, TEST_TIMEOUT },
        { "headless -open examples/uniref50.fa", " -fasta",
            "test_uniref50_out.mfa", true, MINFILESIZE_SMALL, TEST_TIMEOUT },
        { "headless -open examples/uniref50.fa", " -clustal",
            "test_uniref50_out.aln", true, MINFILESIZE_SMALL, TEST_TIMEOUT },
        { "headless -open examples/uniref50.fa", " -msf",
            "test_uniref50_out.msf", true, MINFILESIZE_SMALL, TEST_TIMEOUT },
        { "headless -open examples/uniref50.fa", " -pileup",
            "test_uniref50_out.aln", true, MINFILESIZE_SMALL, TEST_TIMEOUT },
        { "headless -open examples/uniref50.fa", " -pir",
            "test_uniref50_out.pir", true, MINFILESIZE_SMALL, TEST_TIMEOUT },
        { "headless -open examples/uniref50.fa", " -pfam",
            "test_uniref50_out.pfam", true, MINFILESIZE_SMALL, TEST_TIMEOUT },
        { "headless -open examples/uniref50.fa", " -blc",
            "test_uniref50_out.blc", true, MINFILESIZE_SMALL, TEST_TIMEOUT },
        { "headless -open examples/uniref50.fa", " -jalview",
            "test_uniref50_out.jvp", true, MINFILESIZE_SMALL, TEST_TIMEOUT }, };
  }
}
